/*
 * FreeUDFLibC - copyright (c) 1999 Gregory Deatz
 *
 *   math_functions.c
 *     Math routines declarations:
 *       convert_from_33 - convert a string (base33) to an integer
 *       convert_to_33 - convert an integer to base33 string
 *       convert_from_base - convert a string (base<n>) to an integer
 *       convert_to_base - convert an integer to a base<n> string
 *       convert_to_dollar - convert a double to a dollar string ($xxxx.xx)
 *       double_abs - get the absolute value of a double
 *       fixed_point - convert a double to a string with <n> decimal points
 *       integer_abs - get the absolute value of an integer
 *       modulo - perform modulo arithmetic
 *       round - round a double to the nearest integer
 *       truncate - return the integer part of a double
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"
#include "math_functions.h"

static int get_value_of (char, char *);

static int get_value_of (c, digits)
     char c;
     char * digits;
{
  int i;
  
  for (i = 0 ;
       (digits[i] != (char) 0) && (digits[i] != c) ;
       i++) ;
  return i;
}

int convert_from_33 (sz)
     char * sz;
{
  int base = 33;
  
  return convert_from_base (sz, &base, "123456789ABCDEFGHIJKLMNPQRSTUVWXYZ");
}

char * convert_to_33 (value)
     int * value;
{
  int base = 33;
  
  return convert_to_base (value, &base, "123456789ABCDEFGHIJKLMNPQRSTUVWXYZ");
}

int convert_from_base (sz, base, digits)
     char * sz;
     int * base;
     char * digits;
{
  int i;
  int res = 0;
  int len = strlen(sz);
  
  for (i = 0 ; i < len ; i++)
    res = (*base * res) + get_value_of(sz[i], digits);
  return res;
}

char * convert_to_base (value, base, digits)
     int * value;
     int * base;
     char * digits;
{
  char sz_res[LOC_STRING_SIZE + 1];
  char * sz_result = (char *) MALLOC (LOC_STRING_SIZE + 1);
  int i = 0;
  int j, r, len_sz_res;
  int first_time = 1;
  
  for ( j = *value ;
       (j != 0) || first_time;
	j = j / *base, first_time = 0){
    r = j % *base;
    sz_res[i++] = digits[r];
  }
  sz_res[i] = (char) 0;
  sz_result[i] = (char) 0;
  len_sz_res = i;
  
  for (i = 0 ; i < len_sz_res ; i++)
    sz_result[i] = sz_res[len_sz_res - i - 1];
  return sz_result;
}

char * convert_to_dollar (value)
     double * value;
{
  char * sz_result = (char *) MALLOC (LOC_STRING_SIZE + 1);
  
  sprintf(sz_result, "$%.2f", (*value));
    
  return sz_result;
}

double double_abs (value)
     double * value;
{
  if (*value < 0)
    return -*value;
  else
    return *value;
}

char * fixed_point (value, decimals)
     double * value;
     int * decimals;
{
  char * sz_result = MALLOC (LOC_STRING_SIZE + 1);
  
  sprintf(sz_result, "%.*f", *decimals, *value);
    
  return sz_result;
}

int integer_abs (value)
     int * value;
{
  if (*value < 0)
    return -*value;
  else
    return *value;
}

int modulo (numerator, denominator)
     int * numerator;
     int * denominator;
{
  if (*denominator == 0)
    return 0;
  else
    return *numerator % *denominator;
}

int round (value)
     double * value;
{
  if (*value < 0)
    return (int)(*value - 0.5);
  else
    return (int)(*value + 0.5);
}

int truncate (value)
     double * value;
{
  return (int) (*value);
}

