/*
 * FreeUDFLibC - copyright (c) 1999 Gregory Deatz
 *
 *   string_functions.c
 *     String routines:
 *       character - return the character based on the passed integer code.
 *       crlf - return the carriage-return string
 *       encrypt_md5 - hash the passed string using the md5 algorithm
 *       find_word - return the word in a string starting at some index
 *       find_word_index - find the index of a word starting at some index
 *       left - return the leftmost <n> characters of a string
 *       line_wrap - based on lines <n> characters long, return the ith line
 *       l_trim - trim whitespace from the left side of the string
 *       lr_trim - trim whitespace from both sides of the string
 *       mid - return a substring of a string
 *       pad_left - pad the left side of a string with some other string
 *       pad_right - pad the right side of a string with some other string
 *       proper_case - proper case the passed string (every word capitalized)
 *       replace - replace character <a> with character <b> in the string
 *       right - return the rightmost <n> characters of a string
 *       r_trim - Trim the right side of a string of whitespace
 *       string_length - How long is this string?
 *       strip_string - Strip a string of unwanted characters
 *       substr - Find a substring within a string
 *       soundex - return the soundex of a string
 */

#include <stdio.h>
#include <ctype.h>
#include "md5_global.h"
#include "global.h"
#include "md5.h"
#include "string_functions.h"

#define iswordchar(sz, i, start)       \
          (isalnum(sz[i]) ||           \
	   ((i > start) &&             \
	    !isspace(sz[i-1]) &&       \
	    (sz[i] == '\'')))          \

static int find_char PROTO_LIST ((char, char *));

/* Implementations */
char * character (c)
     int * c;
{
  char *sz = (char *) MALLOC (2);
  sz[0] = (char) *c;
  sz[1] = (char) 0;
  
  return sz;
}

char * crlf ()
{
  char * sz = (char *) MALLOC (3);
  sz[0] = (char) 13;
  sz[1] = (char) 10;
  sz[2] = (char) 0;
  return sz;
}

char * encrypt_md5 (sz)
     char * sz;
{
  MD5_CTX context;
  unsigned char digest[16];
  unsigned int len = strlen (sz);
  unsigned int i;
  char * sz_return = (char *) MALLOC (33);

  sz_return[32] = (char) 0;
  
  /* Compute the hash */
  MD5Init (&context);
  MD5Update (&context, sz, len);
  MD5Final (digest, &context);

  for (i = 0; i < 16; i++)
    sprintf (&sz_return[i*2], "%02x", digest[i]);
  return sz_return;
}

char * find_word (sz, index)
     char * sz;
     int * index;
{
  char sz_res[LOC_STRING_SIZE];
  char * sz_result;
  int i;
  int j = 0;
  int len = strlen(sz);
  
  i = find_word_index (sz, index);
  
  while (i < len && j < LOC_STRING_SIZE - 1 &&
	 iswordchar(sz, i, *index)) {
    sz_res[j] = sz[i];
    j++;
    i++;
  }
  sz_res[j] = (char) 0;
  sz_result = (char *) MALLOC(j + 1);
  sprintf(sz_result, "%s", sz_res);
  
  return sz_result;
}

int find_word_index (sz, index)
     char * sz;
     int * index;
{
  char sz_res[LOC_STRING_SIZE];
  int i = *index;
  int j = 0;
  int len = strlen(sz);
  
  while (i < len && j < LOC_STRING_SIZE - 1 &&
	 !iswordchar(sz, i, *index)) {
    sz_res[j] = sz[i];
    j++;
    i++;
  }

  return (i >= len) ? -1 : i;
}

char * left (sz, count)
     char * sz;
     int * count;
{
  int i;
  int len = (strlen(sz) < *count) ? strlen(sz) : *count;
  char * sz_result = (char *) MALLOC (len + 1);
  
  for (i = 0 ; i < len ; i++)
    sz_result[i] = sz[i];
  sz_result[i] = (char) 0;
  
  return sz_result;
}

/*
 * line_wrap algorithm
 * 1. Get the length of the string.
 * 2. If "the rest" of the string is smaller than col_width, just return
 *    it.
 * 3. Otherwise, let 'j' be the end of the column, and decrement j to
 *    a word boundary.
 *    a. Find a word boundary.
 *    b. If j = i (meaning the word is too long), then the word has
 *       has to be chopped.
 */
char * line_wrap (sz, index, col_width)
     char * sz;
     int index, col_width;
{
  int len = strlen(sz); /* (1) */
  int i = index;
  int j;
  char * sz_result;
  
  if ((len - i) <= col_width) /* (2) */
    j = len - i;
  else {
    j = i + col_width - 1;
    while (j > i && !isspace(sz[j]))
      j--; /* (2a) */
    if (j == i)
      j = col_width;
    else
      j = j - i + 1;
  }

  sz_result = (char *) MALLOC(j + 1);
  for (i = 0 ; i < j ; i++)
    sz_result[i] = sz[i + index];

  return sz_result;
}

char * l_trim (sz)
     char * sz;
{
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (len + 1);
  int i;
  
  for (i = 0 ; 
       (i < len) && isspace(sz[i]) ;
       i++) ;

  sprintf(sz_result, "%s", &sz[i]);
  return sz_result;
}

char * lr_trim (sz)
     char * sz;
{
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (len + 1);
  int i, j, k;
  
  for (i = 0 ; 
       (i < len) && isspace(sz[i]) ;
       i++) ;

  for (j = len - 1 ;
       (j >= 0) && isspace(sz[j]) ;
       j--) ;
  
  for (k = i ; k <= j ; k++)
    sz_result[k - i] = sz[k];
  sz_result[k - i] = (char) 0;
  
  return sz_result;
}

char * mid (sz, index, count)
     char * sz;
     int * index;
     int * count;
{
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (*count + 1);
  int i, j, k;

  for (i = *index ; (i < len) && (i - *index < *count) ; i++)
    sz_result[i - *index] = sz[i];
  sz_result[i - *index] = (char) 0;
  
  return sz_result;
}

char * pad_left (sz, sz_pad, size)
     char * sz;
     char * sz_pad;
     int * size;
{
  int len_sz = strlen(sz);
  int len_sz_pad = strlen(sz_pad);
  int len_sz_result = *size > len_sz ? *size : len_sz;
  char * sz_result = (char *) MALLOC (len_sz_result + 1);
  int i, j;
  
  sz_result[len_sz_result] = (char) 0;
  
  for (i = len_sz - 1 ; i >= 0 ; i--)
    sz_result[len_sz_result - (len_sz - i - 1) - 1] = sz[i];

  for (i = len_sz_result - (len_sz - i - 1) - 1 ;
       i >= 0 ;
       ) {
    for (j = len_sz_pad - 1 ;
	 (j >= 0) && (i >= 0) ;
	 j--, i--)
      sz_result[i] = sz_pad[j];
  }
  
  return sz_result;
}

char * pad_right (sz, sz_pad, size)
     char * sz;
     char * sz_pad;
     int * size;
{
  int len_sz = strlen(sz);
  int len_sz_pad = strlen(sz_pad);
  int len_sz_result = *size > len_sz ? *size : len_sz;
  char * sz_result = (char *) MALLOC (len_sz_result + 1);
  int i, j;
  
  sz_result[len_sz_result] = (char) 0;
  
  for (i = 0 ; i < len_sz ; i++)
    sz_result[i] = sz[i];

  while (i < len_sz_result)
    for (j = 0 ;
	 (j < len_sz_pad) && (i < len_sz_result) ;
	 j++, i++)
      sz_result[i] = sz_pad[j];

  return sz_result;
}

char * proper_case (sz)
     char * sz;
{
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (len + 1);
  int i;
  int word_begin = 1;

  for (i = 0 ; i < len ; i++)
    if (isalnum(sz[i]) && word_begin) {
      word_begin = 0;
      sz_result[i] = toupper(sz[i]);
    }
    else if (!isalnum(sz[i])) {
      word_begin = 1;
      sz_result[i] = sz[i];
    } else
      sz_result[i] = sz[i];

  return sz_result;
}

/*
 * replace -
 *   I'm sure there is a *much* more efficient way to do this, but
 *   I don't really care that much because this should be fast enough.
 *   If anybody cares to provide me with some faster/more efficient code,
 *   I'd be glad to include it here.
 */
char * replace (sz, sz_replace, sz_with)
     char * sz;
     char * sz_replace;
     char * sz_with;
{
  int len_sz = strlen(sz);
  int len_sz_replace = strlen(sz_replace);
  int len_sz_with = strlen(sz_with);
  int i, j;
  int k = 0;
  char sz_res[LOC_STRING_SIZE + 1];
  char * sz_result;
  
  for (i = 0 ; i < len_sz ; ) {
    if (strncmp(&sz[i], sz_replace, len_sz_replace) == 0) {
      for (j = 0 ; j < len_sz_with ; j++, k++)
	sz_res[k] = sz_with[j];
      i += len_sz_replace;
    }
    else {
      sz_res[k++] = sz[i];
      i++;
    }
  }
  sz_res[k] = (char) 0;
  sz_result = (char *) MALLOC (strlen(sz_res) + 1);
  sprintf(sz_result, "%s", sz_res);
  return sz_result;
}

char * right (sz, count)
     char * sz;
     int * count;
{
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (*count + 1);
  int i, j;

  for (j = 0, i = (*count > len) ? 0 : len - *count ;
       i < len ; i++, j++)
    sz_result[j] = sz[i];

  sz_result[j] = (char) 0;
  return sz_result;
}

char * r_trim (sz)
     char * sz;
{
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (len + 1);
  int i, j;
  
  for (i = len - 1 ;
       (i >= 0) && isspace(sz[i]) ;
       i--) ;
  
  for (j = 0 ; j <= i ; j++)
    sz_result[j] = sz[j];
  sz_result[j] = (char) 0;
  
  return sz_result;
}

char * soundex (sz)
     char * sz;
{
  int i, j;
  char code;
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (6);
  sz_result[0] = ' ';
  sz_result[1] = '0';
  sz_result[2] = '0';
  sz_result[3] = '0';
  sz_result[4] = '0';
  sz_result[5] = (char) 0;
  
  if (strlen(sz) != 0) {
    sz_result[0] = toupper(sz[0]);

    j = 1;
    i = 1;
    while (i < len && j < 5) {
      switch(sz[i]) {
      case 'B':
      case 'F':
      case 'P':
      case 'V':
      case 'b':
      case 'f':
      case 'p':
      case 'v':
	code = '1'; break;
      case 'C':
      case 'G':
      case 'J':
      case 'K':
      case 'Q':
      case 'S':
      case 'X':
      case 'Z':
      case 'c':
      case 'g':
      case 'j':
      case 'k':
      case 'q':
      case 's':
      case 'x':
      case 'z':
	code = '2'; break;
      case 'D':
      case 'T':
      case 'd':
      case 't':
	code = '3'; break;
      case 'L':
      case 'l':
	code = '4'; break;
      case 'M':
      case 'N':
      case 'm':
      case 'n':
	code = '5'; break;
      case 'R':
      case 'r':
	code = '6'; break;
      default:
	code = '0';
      }
      
      if (code != '0' && code != sz_result[j - 1])
	sz_result[j++] = code;
      i++;
    }
  }
  return sz_result;
}

int string_length (sz)
     char * sz;
{
    return strlen(sz);
}

char * strip_string (sz, sz_strip_chars)
     char * sz;
     char * sz_strip_chars;
{
  int i;
  int j = 0;
  int len = strlen(sz);
  char * sz_result = (char *) MALLOC (len + 1);
 
  for (i = 0 ; i < len ; i++)
    if (!find_char(sz[i], sz_strip_chars)) {
      sz_result[j] = sz[i];
      j++;
    }
  sz_result[j] = (char) 0;
  return sz_result;
}
     
/*
 * substr -
 *   This is most certainly *not* the most efficient way to search for
 *   a string, but I don't really care.
 */
int substr (sz, sz_search)
     char * sz;
     char * sz_search;
{
  int i;
  int result = -1;
  int len_sz = strlen(sz);
  int len_sz_search = strlen(sz_search);
  
  for (i = 0 ; 
       (i < len_sz - len_sz_search + 1) && (result == -1) ;
       i++)
    if (strncmp(&sz[i], sz_search, len_sz_search) == 0)
      result = i;
  
  return result;
}
     
static int find_char (c, sz)
     char c;
     char * sz;
{
  int i = 0;
  int len = strlen(sz);
  
  for (i = 0 ; 
       (i < len) && (c != sz[i]) ;
       i++) ;
  
  if (i == len)
    return 0;
  else
    return -1;
}

